/*
 *  libxcpc by Davide Libenzi (Exception and resource handling in C)
 *  Copyright (C) 2002  Davide Libenzi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include "libxcpc.h"



#define MIN_TEST_TIME 1000000ULL
#define LOOP_COUNT 10000
#define NTLOOP_COUNT 100000



unsigned long long getustime(void) {
	struct timeval tm;

	gettimeofday(&tm, NULL);
	return (unsigned long long) tm.tv_sec * 1000000ULL + (unsigned long long) tm.tv_usec;
}

void thrower(xcpc_ctx ctx, int n) {

	XCPC_THROW(ctx, n, NULL);
}

void non_thrower(xcpc_ctx ctx, int n) {

}

int main (int argc, char **argv) {
	int i;
	unsigned long long ts, te, count = 0, caught = 0;
	xcpc_ctx ctx;

	ctx = xcpc_context_create(NULL);
	ts = getustime();
	do {
		for (i = 0; i < LOOP_COUNT; i++) {
			XCPC_TRY(ctx) {
				thrower(ctx, i);
			}
			XCPC_CATCH_ANY {
				caught++;
			}
			XCPC_END_TRY;
		}
		count++;
		te = getustime();
	} while (te - ts < MIN_TEST_TIME);
	if (caught != count * LOOP_COUNT)
		fprintf(stderr, "exceptions not caught\n");
	fprintf(stdout, "thrower:     %llu us / loop\n", (te - ts) / count);

	count = 0;
	ts = getustime();
	do {
		for (i = 0; i < NTLOOP_COUNT; i++) {
			XCPC_TRY(ctx) {
				non_thrower(ctx, i);
			}
			XCPC_END_TRY;
		}
		count++;
		te = getustime();
	} while (te - ts < MIN_TEST_TIME);
	fprintf(stdout, "non_thrower: %llu us / loop\n", (te - ts) / count);

	xcpc_context_free(ctx);

	return 0;
}

